#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <stdio.h>
#include <X11/Xatom.h>

#define MAX_WINDOW 50

/* windows */
Window windows[MAX_WINDOW];

/* display */
Display *dpy;
int v_screen_num;
Screen *v_screen_ptr;
int v_depth;
XFontStruct *theFont;
Window v_window;
GC     v_gc;
int gc_activated;
/*============================================*/

void waituntilexpose(window)
     Window window;
{
  XEvent report;
  int found;
  /*
    printf("waiting");
  */
  found = 0;
  while ( found == 0) {
    XNextEvent(dpy,&report);
    switch (report.type) {
    case ButtonPress:
      printf(" ButtonPress eventEvent\n");
      break;
    case Expose:
      /*
	printf(".");
	printf("in window %d\n",report.xexpose.window);
      */
      if(report.xexpose.window == window) found = 1;
      break;
    default:
      printf(" Unknown event type= %d\n",report.type);
      break;
    } /* End switch record.type */
  } /* end while */
  /*
    printf("\n");
  */
} /* end waituntilexpose */
/*============================================*/

void winit_(int *ier, int *width, int*height, int *requestsize)
{
  char *display_name = NULL;
  char *fontname="9x15";
  dpy = XOpenDisplay(display_name);
  if(dpy == NULL) 
    {
      /*      printf("cannot connect to server\n"); */
      *ier = 1;
      return;
    }
  *ier = 0;
  v_screen_num = DefaultScreen(dpy);
  v_screen_ptr = DefaultScreenOfDisplay(dpy);
  v_depth = DefaultDepth(dpy,v_screen_num);
  *width = WidthOfScreen(v_screen_ptr);
  *height = HeightOfScreen(v_screen_ptr);
  if ( ( theFont = XLoadQueryFont(dpy,fontname) ) == NULL)
    {
      (void) fprintf(stderr, "\nCannot open font %s\n",fontname);
      exit(-1);
    }
  *requestsize = XMaxRequestSize(dpy);
  XSynchronize(dpy,0);
  gc_activated = 0;
} /* end w_init_ */
/*============================================*/

void wopen_(int *aw, int *x, int*y,unsigned int *w, unsigned int*h, int *fg, int *bg)
{
  Visual *visual;
  XSetWindowAttributes xswda;
  XGCValues xgcv;
  unsigned long valuemask;
  unsigned int borderwidth=8;
  /*Font font;*/
  
  /*
    (void) fprintf (stdout,"Aw=%d x=%d y=%d w=%d h=%d fg=%d bg=%d\n",
    *aw,*x,*y,*w,*h,*fg,*bg);
    */
  visual = DefaultVisualOfScreen(v_screen_ptr);
  xswda.backing_store = Always;
  xswda.background_pixel = *bg;
  xswda.event_mask = ExposureMask;
  valuemask = CWEventMask | CWBackPixel | CWBackingStore;
  
  v_window = XCreateWindow(dpy,RootWindowOfScreen(v_screen_ptr),*x,*y,
			   *w,*h,borderwidth,v_depth,InputOutput,visual,valuemask,&xswda);
  /*
    (void) fprintf (stdout,"windowid=%d\n",v_window);
  */
  if (! gc_activated)
    { 
      xgcv.foreground = *fg;
      xgcv.background = *bg;
      xgcv.font = theFont->fid;
      valuemask = GCForeground | GCBackground | GCFont;
      /*
	(void) fprintf (stdout,"create GC\n");
      */
      v_gc = XCreateGC(dpy,v_window,valuemask,&xgcv);
      gc_activated = 1;
    }
  /*
    (void) fprintf (stdout,"map window\n");
  */
  windows[*aw-1] = v_window;
  /* vreemd maar waar de volgende regel */
  XMoveWindow(dpy,v_window,*x,*y); 
  XMapWindow(dpy,v_window);
  waituntilexpose(v_window);
} /* w_open */
/*============================================*/

void wsubopen_(int *aw, int *x, int*y,unsigned int *w, unsigned int*h, unsigned int*borderwidth, int *fg, int *bg)
{
  Visual *visual;
  XSetWindowAttributes xswda;
  XGCValues xgcv;
  unsigned long valuemask;
  Window s_window;
  /*Font font;*/
  
  /*
    (void) fprintf (stdout,"Aw=%d x=%d y=%d w=%d h=%d fg=%d bg=%d\n",
    *aw,*x,*y,*w,*h,*fg,*bg);
    */
  visual = DefaultVisualOfScreen(v_screen_ptr);
  xswda.backing_store = Always;
  xswda.background_pixel = *bg;
  xswda.event_mask = ExposureMask;
  valuemask = CWEventMask | CWBackPixel | CWBackingStore;
  
  s_window = XCreateWindow(dpy,v_window,*x,*y,
			   *w,*h,*borderwidth,v_depth,InputOutput,visual,valuemask,&xswda);
  /*
    (void) fprintf (stdout,"windowid=%d\n",s_window);
  */
  if (! gc_activated)
    { 
      xgcv.foreground = *fg;
      xgcv.background = *bg;
      xgcv.font = theFont->fid;
      valuemask = GCForeground | GCBackground | GCFont;
      /*
	(void) fprintf (stdout,"create GC\n");
      */
      v_gc = XCreateGC(dpy,s_window,valuemask,&xgcv);
      gc_activated = 1;
    }
  /*
    (void) fprintf (stdout,"map window\n");
  */
  windows[*aw-1] = s_window;
  v_window = s_window;
  XMapWindow(dpy,s_window);
  /* vreemd maar waar de volgende regel */
  /*
    XMoveWindow(dpy,s_window,*x,*y); 
  */
  waituntilexpose(s_window);
} /* w_subopen */
/*============================================*/


void gcxor_(void)
{
  XGCValues xgcv;
  xgcv.function = GXxor;
  XChangeGC(dpy,v_gc,GCFunction,&xgcv);
} 
/*============================================*/

void gcdefault_(void)
{
  XGCValues xgcv;
  xgcv.function = GXcopy;
  XChangeGC(dpy,v_gc,GCFunction,&xgcv);
} 
/*============================================*/

void wsync_(void)
{
  XSync(dpy,True);
} 
/*============================================*/

void wwindowid_(int *aw, int *v)
{
  *v= windows[*aw-1];
} /* wwindowid */
/*============================================*/

void wgetcolour_(char *colourname, int *c)
{
  Colormap cmap;
  XColor colorcell_def;
  XColor rgb_db_def;
  cmap = XDefaultColormapOfScreen(v_screen_ptr);
  if (XAllocNamedColor(dpy,cmap,colourname, &colorcell_def, &rgb_db_def) == 0)
    { 
      /*
      (void) fprintf(stderr, "\nCannot load color %s\n",colourname);
      */
      *c = -1;
    }
  else
    {
      *c = (int)colorcell_def.pixel;
      if((colorcell_def.red-rgb_db_def.red) != 0 ||
	 (colorcell_def.green-rgb_db_def.green) != 0 || 
	 (colorcell_def.blue-rgb_db_def.blue) != 0 )
	{
	  printf("%s ",colourname);
	  printf("%i %i %i ",colorcell_def.red,colorcell_def.green,colorcell_def.blue);
	  printf("%i %i %i\n",rgb_db_def.red,rgb_db_def.green,rgb_db_def.blue);
        }
      
    }
} /* end w_get_colour */
/*============================================*/

void wsetcolour_(int  *c)
{
  unsigned long cl = (unsigned long)*c;
  XSetForeground(dpy,v_gc,cl);
} /* end w_set_colour_ */
/*============================================*/

void wactivate_(int *aw)
{
  v_window = windows[*aw-1];
} /* w_activate */
/*============================================*/

void wtitle_(char *title)
{
  XStoreName(dpy,v_window,title);
} /* w_title */
/*============================================*/

void wdrawline_(int *x1, int *y1, int *x2, int *y2)
{
  XDrawLine(dpy,v_window,v_gc,*x1,*y1,*x2,*y2);
} /* w_draw_line */
/*============================================*/

void wdrawpoint_(int *x, int *y)
{
  XDrawPoint(dpy,v_window,v_gc,*x,*y);
} /* w_draw_point */
/*============================================*/

void wflush_(void)
{
  XFlush(dpy);
} /* w_flush */
/*============================================*/

void wclear_(void)
{
  XClearWindow(dpy,v_window);
} /* v_clear */
/*============================================*/

void wend_(void)
{
  XCloseDisplay(dpy);
} /* w_end */
/*============================================*/

void wdrawcenteredstring_(int *x, int *y, int *ls, char *s)
{
  int direction;
  int ascent;
  int descent;
  XCharStruct overall;
  int hor;
  int ver;
  XTextExtents(theFont,s,*ls,&direction,&ascent,&descent,&overall);
  ver = (overall.ascent+overall.descent)/2;
  hor = overall.width/2;
  /*  printf("%d %d\n",hor,ver); */
  XDrawString(dpy,v_window,v_gc,*x-hor,*y+ver,s,*ls);
} 
/*============================================*/

void wdrawstring_(int *x, int *y, int *ls, char *s)
{
  XDrawString(dpy,v_window,v_gc,*x,*y,s,*ls);
} 
/*============================================*/

void wclose_(int *aw)
{  
  XUnmapWindow(dpy,windows[*aw-1]);
  XDestroyWindow(dpy,windows[*aw-1]);
} /* w_close */
/*============================================*/

void wdrawpoints_(int *np,XPoint *xy)
{
  /*
    int i;
    (void) fprintf(stdout, "np = %d\n",*np);
    for (i=0;i<*np;i++) (void) fprintf(stdout,"i=%d x=%d y=%d\n",
    i,xy[i].x,xy[i].y);
  */
  XDrawPoints(dpy,v_window,v_gc,xy,*np,CoordModeOrigin);
} /* w_draw_points */
/*============================================*/

void wdrawlines_(int *np,XPoint *xy)
{
  /*
    int i;
    (void) fprintf(stdout, "np = %d\n",*np);
    for (i=0;i<=*np-1;i++) (void) fprintf(stdout,"i=%d x=%d y=%d\n",
    i,xy[i].x,xy[i].y);
  */
  XDrawLines(dpy,v_window,v_gc,xy,*np,CoordModeOrigin);
} /* w_draw_lines */
/*============================================*/

void wfillpolygon_(int *np,XPoint *xy)
{
  /*
    int i;
    (void) fprintf(stdout, "np = %d\n",*np);
    for (i=0;i<=*np-1;i++) (void) fprintf(stdout,"i=%d x=%d y=%d\n",
    i,xy[i].x,xy[i].y);
  */
  XFillPolygon(dpy,v_window,v_gc,xy,*np,Complex,CoordModeOrigin);
} /* w_fill_polygon */
/*============================================*/

void wdrawarc_(int *x, int *y,unsigned int *w,unsigned int *h, int *a1, int *a2)
{
  XDrawArc(dpy,v_window,v_gc,*x,*y,*w,*h,*a1,*a2);
} /* w_draw_arc */
/*============================================*/

void wfillarc_(int *x, int *y,unsigned int *w,unsigned int *h, int *a1, int *a2)
{
  XFillArc(dpy,v_window,v_gc,*x,*y,*w,*h,*a1,*a2);
} /* w_draw_arc */
/*============================================*/

void wmove_(int *x,int *y)
{
  XMoveWindow(dpy,v_window,*x,*y);
}
void wresize_(unsigned int *w, unsigned int *h)
{
  XResizeWindow(dpy,v_window,*w,*h);
}
void wmoveresize_(int *x,int *y, unsigned int *w, unsigned int *h)
{
  XMoveResizeWindow(dpy,v_window,*x,*y,*w,*h);
}
/*============================================*/

void PrintGravity(int g)
{
  if(g == ForgetGravity)
    printf("Forget");
  else if(g == StaticGravity)
    printf("Static");
  else if(g == CenterGravity)
    printf("Center");
  else if(g == NorthWestGravity)
    printf("NorthWest");
  else if(g == NorthGravity)
    printf("North");
  else if(g == NorthEastGravity)
    printf("NorthEast");
  else if(g == EastGravity)
    printf("East");
  else if(g == SouthEastGravity)
    printf("SouthEast");
  else if(g == SouthGravity)
    printf("South");
  else if(g == SouthWestGravity)
    printf("SouthWeast");
  else if(g == WestGravity)
    printf("West");
} /* PrintGravity */
/*============================================*/

void PrintEventMask(long e)
{     
  if(e & KeyPressMask) printf(" KeyPress");
  if(e & KeyReleaseMask) printf(" KeyRelease");
  if(e & ButtonPressMask) printf(" ButtonPress");
  if(e & ButtonReleaseMask) printf(" ButtonRelease");
  if(e & EnterWindowMask) printf(" EnterWindow");
  if(e & LeaveWindowMask) printf(" LeaveWindow");
  if(e & PointerMotionMask) printf(" PointerMotion");
  if(e & PointerMotionHintMask) printf(" PointerMotionHint");
  if(e & Button1MotionMask) printf(" Button1Motion");
  if(e & Button2MotionMask) printf(" Button2Motion");
  if(e & Button3MotionMask) printf(" Button3Motion");
  if(e & Button4MotionMask) printf(" Button4Motion");
  if(e & Button5MotionMask) printf(" Button5Motion");
  if(e & ButtonMotionMask) printf(" ButtonMotion");
  if(e & KeymapStateMask) printf(" KeymapState");
  if(e & ExposureMask) printf(" Exposure");
  if(e & VisibilityChangeMask) printf(" VisibilityChange");
  if(e & StructureNotifyMask) printf(" StructureNotify");
  if(e & ResizeRedirectMask) printf(" ResizeRedirect");
  if(e & SubstructureNotifyMask) printf(" SubstructureNotify");
  if(e & SubstructureRedirectMask) printf(" SubstructureRedirect");
  if(e & FocusChangeMask) printf(" FocusChange");
  if(e & PropertyChangeMask) printf(" PropertyChange");
  if(e & ColormapChangeMask) printf(" ColormapChange");
  if(e & OwnerGrabButtonMask) printf(" OwnerGrabButton");
} /* PrintEventMask */
/*============================================*/
void wgetgeom_(int *x, int *y, unsigned int *w, unsigned int *h, unsigned int *b)
{
  unsigned int d;
  Window root;
  
  XGetGeometry(dpy,v_window,&root,x,y,w,h,b,&d);
} /* wgetgeom */
/*============================================*/

void winfo_(int *i)
{
  Window win, root;
  int x, y;
  unsigned int w, h, b, d;
  XWindowAttributes wa;
  
  win = windows[*i-1];
  printf("window %d id=%ld",*i,win);
  if (win == v_window ) printf(" Active Window");
  printf("\n  ");      
  XGetGeometry(dpy,win,&root,&x,&y,&w,&h,&b,&d);
  
  printf("Geom:x=%d y=%d width=%d height=%d",x,y,w,h);
  printf(" root=%ld",root);
  printf(" border_width=%d",b);
  printf(" depth=%d",d);
  printf("\n  ");
  XGetWindowAttributes(dpy,win,&wa);
  printf("Attrib:x=%d y=%d width=%d height=%d",wa.x,wa.y,wa.width,wa.height);
  printf(" border_width=%d",wa.border_width);
  printf(" depth=%d",wa.depth);
  printf("\n  ");
  printf("class=");
  if(wa.class == InputOutput) 
    printf("InputOutput");
  else if(wa.class == InputOnly) 
    printf("InputOnly");
  else
    printf("unknown[%d]",wa.class);
  printf(" bit_gravity=");
  PrintGravity(wa.bit_gravity);
  printf(" win_gravity=");
  PrintGravity(wa.win_gravity);
  printf("\n  ");
  printf("backing_store=");
  if(wa.backing_store == NotUseful)
    printf("NotUseful");
  else if(wa.backing_store == WhenMapped)
    printf("WhenMapped");
  else if(wa.backing_store == Always)
    printf("Always");
  printf(" backing_planes=%ld",wa.backing_planes);
  printf(" backing_pixel=%ld",wa.backing_pixel);
  printf("\n  ");
  printf("save_under=%d",wa.save_under);
  printf(" map_state=");
  if(wa.map_state == IsUnmapped)
    printf("IsUnmapped");
  else if(wa.map_state == IsUnviewable)
    printf("IsUnviewable");
  else if(wa.map_state == IsViewable)
    printf("IsViewable");
  if(wa.all_event_masks)
    {
      printf("\n  ");
      printf("all_event_masks=");
      PrintEventMask(wa.all_event_masks);
    }
  if(wa.your_event_mask)
    {
      printf("\n  ");
      printf("your_event_mask=");
      PrintEventMask(wa.your_event_mask);
    }
  if(wa.do_not_propagate_mask)
    {
      printf("\n  ");
      printf("do_not_propagate_mask=");
      PrintEventMask(wa.do_not_propagate_mask);
    }
  printf("\n");
} /* winfo */
/*============================================*/

void wdisplayinfo_(void)
{
  printf("display");
  printf("\n  ");
  if(DoesBackingStore(v_screen_ptr)) printf (" DoesBackingStore");
  if(DoesSaveUnders(v_screen_ptr)) printf (" DoesSaveUnders");
  printf(" Height %d pixels",DisplayHeight(dpy,v_screen_num));
  printf(" %d mm",DisplayHeightMM(dpy,v_screen_num));
  printf(" Width %d pixels",DisplayWidth(dpy,v_screen_num));
  printf(" %d mm",DisplayWidthMM(dpy,v_screen_num));
  printf(" RootWindowOdScreen=%ld",RootWindowOfScreen(v_screen_ptr));
  printf("\n");
}
/*============================================*/
